{
* Autor.......: TVS-Soft, Thomas von Stetten (CIS-ID: 100063,240)
* Version.....: 1.0 vom 03.06.95
* Beschreibung: TGAUGE95 ist eine Gauge-Komponente, die so aussieht, wie
*               die von MS in OFFICE und WIN95 verwendeten.
* Anwendung...:
*   - Instanz der Komponente erzeugen
*   - Eigenschaften setzen:
*     .BarColor:  Farbe der Kstchen (Spots), die den Balken Darstellen
*                 (Default: clNavy)
*     .CurrValue: Aktueller Wert des Balkens. Dies ist NICHT die Anzahl
*                 Punkte auf dem Balken, sondern der interne Zhler.
*                 (Default: Im Designmodus hat dieses Property den
*                           Standard-Wert 100, damit man das Ergebnis sehen
*                           kann. Sonst wird es mit 0 initialisiert.)
*     .HSpace:    Platz in der Horizontalen zwischen dem Rahmen und den
*                 Spots sowie zwischen zwei Spots. (Default: 3)
*     .MinValue:  Minimalwert der Gauge. (Default: 0)
*     .MaxValue:  Maximalwert des Gauge-Zhlers (Default: 100)
*     .Spots:     Gibt die Anzahl Punkte an, die maximal angezeigt werden.
*                 (Default: 20)
*     .Steps:     Anzahl schritte von Minvalue zu MaxValue. Daraus wird
*                 dann bei .INCREMENT die Gre eines Schrittes berechnet.
*                 (Default: 100)
*     .VSpace:    Platz in der Vertikalen zwischen dem Rahmen und den
*                 Spots. (Default: 3)
*     .Width:     Beim Einstellen der Breite wird der angegebene Wert so
*                 umgerechnet, da die angegebenen Spots korrekt auf diese
*                 Breite verteilt werden knnen. Dies ist notwendig, weil
*                 die Spots immer die selbe Breite haben und auch der
*                 Abstand zwischen 2 Spots immer gleich gro ist. Dazu
*                 habe ich das Standard-Property WIDTH berschrieben. Es
*                 funktioniert aber immer noch nicht in allen Situationen,
*                 weil ich die Komponente nicht unntig aufblasen wollte.
*                 Beispielsweise wird die Breite nicht angepat, wenn man
*                 den 3D-Effekt (BEVELxxx) ndert. Aus diesem Grund
*                 sollten Sie, wenn Sie die Bevel-Werte ndern, immer
*                 am Ende nochmal "Width := Width" setzen!
*                 Die Breite wird NICHT angepat, wenn ein ALIGN-Wert fr
*                 die Komponente angegeben wurde. (Default: 267)
*
*   - Gaugewert periodisch mit .INCREMENT verndern lassen. Dies ist dann
*     sinnvoll, wenn man beispielsweise eine Datei Satzweise durcharbeitet
*     und nach jedem Satz ein .Increment aufrufen.
*   - Gaugewert direkt mit Currvalue := <nn> verndern. Dies ist dann
*     sinnvoll, wenn man den genauen Verlauf einer Aktion nicht
*     beeinflussen kann. Beispielsweise bei einem Installationsporgramm,
*     bei dem man zwar wei, da insgesamt <xy> Bates kopiert werden, aber
*     ein Update nur einmal pro Datei machen kann. Dann setzt man den
*     aktuellen Wert einfach nach dem kopieren auf
*      <CurrValue := CurrValue + FileSize;>
*
**************************************************************************
*  Die Komponente ist natrlich Freeware. Wenn jemand Fehler findet oder
*  sonstige Anregungen hat, bitte ber CIS an mich.
*
*  Die Benutztung geschieht auf eigene Gefahr. Ich kann weder fr das
*   korrekte Arbeiten der Komponente noch dafr, da Sie euren
*   Anforderungen gengt, irgend eine Art von Garantie bernehmen.
**************************************************************************}
{DEFINE DEBUG}                      {Zum Test einfach das $-Zeichen einsetzen}
{$IFNDEF DEBUG}                     {Wenn nicht Test, dann ohne Debug-Infos}
{$D-;$L-;$Y-}                       {Kompilieren}
{$ELSE}
{$Q+}                               {berlaufprfung (gell Kai-Uwe!)}             
{$ENDIF}

unit Gauge95;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TSpotCount = 1..65500;             {Mindestens immer 1 Spot!}
  TStepCount = 1..2147483647;        {Wert darf nicht 0 sein!}

  TGauge95 = class(TCustomPanel)
  private
    {Private-Declaration}
    Procedure PaintSpots;            {Alle ausstehenden Spots anzeigen}
  protected
    FCurrValue:    Longint;          {Aktueller Wert der Gauge}
    FMaxValue:     Longint;          {Maximaler Wert}
    FMinValue:     Longint;          {Minimaler Wert}
    FSpots:        TSpotCount;       {Anzahl Punkte bei Maximalanzeige}
    FHSpace:       Word;             {Zwischenraum zwischen 2 Spots}
    FVSpace:       Word;             {Zwischenraum zwischen Rahmen und Spots}
    FSteps:        TStepCount;       {Einzelschritte fr Autoincrement}
    FBarColor:     TColor;           {Farbe fr die Punkte}
    FSpotsVisible: Byte;             {Wieviele Dots sind schon sichtbar}
    FWidth:        Integer;          {Private Kopie von Width fr Berechnete Werte}

    Procedure SetCurrValue(Value: LongInt);
    Procedure SetMaxValue(Value: LongInt);
    Procedure SetMinValue(Value: LongInt);
    Procedure SetSpots(Value: TSpotCount);
    Procedure SetSteps(Value: TStepCount);
    Procedure SetBarColor(Value: TColor);
    Procedure SetVSpace(Value: Word);
    Procedure SetHSpace(Value: Word);
    procedure Paint; override;
    Function CalcBorderWidth: SmallInt;
    Procedure SetWidth(Value: Integer);

  public
    Constructor Create(AOwner: TComponent); override;
    Procedure Increment;             {Einen Schritt (<Steps>) Vor}

  published
    property Align;
    property BarColor: TColor   Read FBarColor  Write SetBarColor default clNavy;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property Color;
    property Ctl3D;
    property MinValue: LongInt  Read FMinValue  Write SetMinValue  Default 0;
    property MaxValue: LongInt  Read FMaxValue  Write SetMaxValue  Default 100;
    property CurrValue: LongInt Read FCurrValue Write SetCurrValue
        Default 100;{Damit die 100 aus dem Designmode nicht gespeichert werden}
    Property Steps: TStepCount  Read FSteps     Write SetSteps     Default 100;
    Property Spots: TSpotCount  Read FSpots     Write SetSpots     Default 20;
    Property VSpace: Word       Read FVSpace    Write SetVSpace    Default 3;
    Property HSpace: Word       Read FHSpace    Write SetHSpace    Default 3;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property Width: Integer     Read FWidth Write SetWidth; {Orig. berladen!}
  end;

procedure Register;

implementation

{************************************************************************}
{Erzeugen und Eigenschaften initialisieren}
constructor TGauge95.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csOpaque];
  Caption      := '';         {Default-Caption berschreiben!!!}
  TabStop      := False;      {Kein Tabstop auf dieser Komponente}
  FBarColor    := clNavy;
  BevelOuter   := bvLowered;
  FMaxValue    := 100;
  FSpots       := 20;
  FSteps       := 100;
  FHSpace      := 3;
  FVSpace      := 3;
  Width        := 267;        {Erst hier wegen Berechnung in SetWidth()}
  Height       := 20;

  {Damit man im Designer auch was sieht}
  if (csDesigning in ComponentState) then FCurrValue := FMaxValue;
end;

{Komplett neu zeichnen}
procedure TGauge95.Paint; {Override}
begin
  if not (Parent = NIL) then begin
    Inherited Paint;
    FSpotsVisible := 0;           {Komplett neu zeichnen}
    PaintSpots;                   {Spots anzeigen}
  end;
end;

{Alle bentigten Spots zeichnen}
Procedure TGauge95.PaintSpots;
var iCnt:          Integer;       {Laufvariable zum Zeichen}
    iFirst:        Integer;       {Nummer des 1. zu zeichnenden Spots}
    iLast:         Integer;       {Nummer des letzen zu zeichnenden Spots}
    SpotWidth:     Word;          {Breite eines Spots; wird auch zum Verschieben benutzt}
    BorderSize:    SmallInt;      {Breite des 3D-Rands des Panels}
    Rect:          TRect;         {Rechteck, das als Spot gezeichnet wird}
begin
  iLast := Trunc((FCurrValue - FMinValue)
               / ((FMaxValue - FMinValue) / FSpots)
               + 0.00000001);     {Ohne das gehts irgendwie nicht immer}
                                  {Bsp: Trunc((100-0)/((100-0)/_21_) }
                                  {   = 20, sollte aber 21 sein}

  {Nur zeichnen, wenn Anzeige gendert und das Ding auch sichtbar ist}
  {  Zusatzbedingungen sind ntig, weil sonst Probleme, weil diese Proc}
  {  auch von einigen Write-Methoden aufgerufen wird.}
  if (iLast > FSpotsVisible) AND (Visible) AND (Parent <> NIL) then begin
    iFirst     := FSpotsVisible + 1;

    {Rand fr 3D-Effekt des Panels berechnen}
    BorderSize := CalcBorderWidth;
    Rect.Top   := BorderSize + FVSpace;
    Rect.Left  := BorderSize + FHSpace;  {Quasi der Offset}

    BorderSize := BorderSize * 2;
    {Inherited Width, wegen Alignment!!!}
    SpotWidth  := ((inherited Width) - BorderSize - FHSpace) div FSpots;

    Rect.Bottom := Rect.Top + (Height - BorderSize - FVSpace - FVSpace);
    Rect.Left   := Rect.Left + ((iFirst-2) * SpotWidth);
    Rect.Right  := Rect.Left + SpotWidth - FHSpace; {Leerraum}

    with Canvas do begin
      Brush.Color := FBarColor;
      Brush.Style := bsSolid;
      for iCnt := iFirst to iLast do begin
        Inc(Rect.Left,  SpotWidth);    {Linken Rand auf den Anfang des nchsten }
        Inc(Rect.Right, SpotWidth);
        FillRect(Rect);                {Rechnteck zeichnen}
      end;
    end;
    FSpotsVisible := iLast;            {Merken, wieviele schon angezeigt sind}
  end;
end;

Procedure TGauge95.SetWidth(Value: Integer);
var iBorder: Integer;
begin
  if Align <> alNone then       {Wenn Alignment definiert,}
    inherited Width := Value    {  dann einfach setzen}
  else begin                    {wenn freie Zuordnung mglich, dann anpassen}
    iBorder := (2 * CalcBorderWidth) {Linker und rechter Rand}
             + FHSpace;              { + erster Freiraum links}
    inherited Width := Trunc(((Value - iBorder) / FSpots) + 0.5)
                     * FSpots
                     + iBorder;
  end;
  FWidth := inherited Width     {Lokale Variable aktualisieren}
end;

{Farbe des Balkens setzen}
procedure TGauge95.SetBarColor(Value: TColor);
begin
  if Value <> FBarColor then begin
    FBarColor := Value;
    Refresh;
  end;
end;

{Minimal-Wert setzen}
Procedure TGauge95.SetMinValue(Value: LongInt);
begin
  if   (FMinValue <> Value)            {Anderer Wert als bisher}
   AND (FMaxValue <> Value) then begin {Division durch 0 in <PaintSpots>}
    FMinValue := Value;
    if FCurrValue < FMinValue then
      FCurrValue := MinValue;          {neu setzen mit Refresh}

    Refresh;
  end;
end;

{Maximalwert setzen}
Procedure TGauge95.SetMaxValue(Value: LongInt);
begin
  if   (FMaxValue <> Value)            {Anderer Wert als bisher}
   AND (FMinValue <> Value) then begin {Division durch 0 in <PaintSpots>}
    FMaxValue := Value;
    if FCurrValue > FMaxValue then
      FCurrValue := FMaxValue;         {neu setzen mit Refresh}

    Refresh;
  end;
end;

{Aktuellen Wert setzen}
Procedure TGauge95.SetCurrValue(Value: LongInt);
begin
  {Wenn neuer Wert grer als der Bisherige, dann nicht}
  {  komplett neu, sondern nur den genderten Teil ausgeben}
  if Value > FCurrValue then begin
    {Grenzen beachten!}
    if Value > FMaxValue then FCurrValue := FMaxValue
    else                      FCurrValue := Value;
    PaintSpots;
  end
  {Wenn der neue Wert kleiner ist, als der aktuelle Wert,}
  {  dann komplett neu zeichnen, um den Hintergrund zu lschen}
  else if Value < FCurrValue then begin
    {Grenzen beachten!}
    if Value > FMinValue then FCurrValue := FMinValue
    else                      FCurrValue := Value;
    Refresh;
  end;
end;

{Anzahl Einzelschritte fr <Increment> setzen}
Procedure TGauge95.SetSteps(Value: TStepCount);
begin
  if Value <> FSteps then begin
    FSteps := Value;
    Refresh;
  end;
end;

{Anzahl Punkte setzen}
Procedure TGauge95.SetSpots(Value: TSpotCount);
begin
  if Value <> FSpots then begin
    FSpots := Value;
    SetWidth(FWidth);           {Breite neu berechnen}
    Refresh;
  end;
end;

{Vertikaler Abstand zwischen Rahmen und Spots}
Procedure TGauge95.SetVSpace(Value: Word);
begin
  if Value <> FVSpace then begin
    FVSpace := Value;
    Refresh;
  end;
end;

{Horizontaler Abstand zwischen Rahmen und Spots}
{  sowie Abstand zwischen 2 Spots}
Procedure TGauge95.SetHSpace(Value: Word);
begin
  if Value <> FHSpace then begin
    FHSpace := Value;
    SetWidth(FWidth);           {Breite neu berechnen}
    Refresh;
  end;
end;

{Berechnung der Breite des 3D-Rahmens des Panels}
Function TGauge95.CalcBorderWidth: SmallInt;
begin
  Result := BorderWidth;
  if BevelOuter <> bvNone then   Inc(Result, BevelWidth);
  if BevelInner <> bvNone then   Inc(Result, BevelWidth);
end;

{Erhht den aktuellen Wert um eine Einheit - entsprechend <Steps>}
Procedure TGauge95.Increment;
begin
  if FCurrValue < FMaxValue then begin
    Inc(FCurrValue, trunc(((FMaxValue - FMinValue) / FSteps) + 0.5));
    if FCurrValue > FMaxValue then FCurrValue := FMaxValue;
    PaintSpots;
  end;
end;

{************************************************************************}
procedure Register;
begin
  RegisterComponents('Beispiele', [TGauge95]);
end;

end.
